/*{ Template_FreeRTOS::main.c ********************************************
 
 * (w) 20100719 by Gregor Rebel
 
 * Basic framework template of blinking LED for
     - Olimex STM32-P103 eval board
     - StdPeripheralsLibrary CMSIS V3.4.0 (tested)
     - GNU toolchain from CodeSourcery G++
     - local/ remote debugging (JTAG-Adapter may be connected to different computer)


 * Coding-style
    + gnu99
      It's year 2011, get a decent compiler if your's does not comply with this.
      arm-none-eabi-gcc-4.4.1 does support gnu99.
      
    + Text Folding
      This source makes use of text folding to improve hierarchical readability 
      It has been written with jEdit + plugin Configurable Fold Handler.
      Activate folding in your favourite text-editor for { and } 
      (fold-start and -end) for best visual experience.


 * Compile:
   ./compile.sh
   or
   make all

 * Flash onto u-controller:
   - change _openocd_flash.sh according to your board and adapter
   - flash binary
     ./_/openocd_flash.sh main.bin
   
 * Debug:
   - change openocd_gdb.sh and gdb.sh according to your board and adapter
   - start openocd as gdb-server (may even run on another computer connected over network)
     ./_/openocd_gdb.sh
   - start gdb text user-interface in another shell
     ./_/gdb.sh
     
         

 * Configuration of source-code:
    + stm32f10x_conf.h
      - modify according to your hardware (if differing from above) 
    
    + stm32f10x_it.c .
      - implement your own interrupthandlers in existing functions
      
    + main.c (this file)
      - extend as you like
      
    + makefile
      - all .c, .h and .s files in local directory are compiled and linked together automatically.
      
      
 * Prerequisites:
   + OpenOCD
     - configure & run  install/install_OpenOCD.sh
     
   + CodeSourcery G++
     - configure & run  install/install_SourceryG++.sh
     - add to PATH (replace <XXX>according to your local filesystem):
       export PATH="<XXX>/SourceryG++/arm-2010q1/bin/:<XXX>/SourceryG++/arm-2010q1/libexec/gcc/arm-none-eabi/4.4.1/:$PATH"
 
   + StdPeripheralsLibrary CMSIS V3.3.0
     - configure & run  install/install_STM-StdPeripheralsLibrary.sh
     - create link named StdPeripheralsLibrary to subfolder Libraries/ of unpacked archive 

    cp StdPeripheralsLibrary/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x_orig.h
    chmod +w Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h
       - move assert-macro from stm32f10x_conf.h into stm32f10x.h
       - source: http://www.mikrocontroller.net/topic/156063#1472643

    cp StdPeripheralsLibrary/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md.s Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md_orig.s
    chmod +w Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md.s
      - change line (ResetHandler Bug: http://www.mikrocontroller.net/topic/156063#1476546):
        .word	Reset_Handler
     -> .word	Reset_Handler + 1
   
  * Scripts:
    Some helper scripts can be found inside scripts/.
    Each script-name starts with a comma to make it easy to tab them after adding scripts/ to $PATH.
    
    clean.sh           removes all object files in current and all subfolders
    compile.sh         cleans, compiles and flashes application 
    cscope.sh          creates a searchable index of all .c and .h files (jEdit provides cscope-plugin)
    findInLibrary.sh   searches for given text in all .h files inside StdPeripheralsLibrary
    findInSources.sh   searches for given text in all .h files of given folder
    flash.sh           kills running openocd + starts openocd_flash.sh
    gdb.sh             starts remote gdb-session (change according to your setup)
    jedit.sh           wrapper for cscope to use jEdit as Source-Browser (download from jedit.org)
                      Issue before starting cscope: export CSCOPE_EDITOR=`pwd`/scripts/jedit.sh
    openocd_flash.sh   starts openocd to flash given binary onto uC 
    openocd_gdb.sh     starts openocd as GDB server waiting for a GDB to connect
    paths.sh           can be sourced to provide all required paths (adapt to your setting before)
                       Usage: add a line similar to this to your ~/.bashrc
                       source $HOME/Source/STM32_Template/scripts/paths.sh

}*/
//{ Defines/ TypDefs *****************************************************

#define PB_LED1   GPIOC, 6    // status led 1 on Olimex proto board P107

//} Defines
//{ Includes *************************************************************

/**
 * IO definitions
 *
 * define access restrictions to peripheral registers
 *
 * (Copied from core_cm3.h because its required to include stm32f10x.h which is required t include core_cm3.h!)
 */

 // Basic set of helper functions
#include "stm32_io.h"

#include <stdlib.h>

#define USE_STDPERIPH_DRIVER
#include "stm32f10x.h"
#include "StdPeripheralsLibrary/CMSIS/CM3/CoreSupport/core_cm3.h"
#include "stm32f10x_conf.h"

// ADC
#include "stm32f10x_adc.h"
#include "stm32f10x_dma.h"

// Timers
#include "stm32f10x_tim.h"

#include "stm32f10x_it.h"
#include "system_stm32f10x.h"
#include "bits.h"

#include <string.h>
#include "FreeRTOS.h"
#include "task.h"


//} Includes
//{ Structures/ Enums ****************************************************

typedef struct {
  GPIO_TypeDef* GPIOx;   // which port to use for LED
  u32_t Pin;             //
  u16_t OnPeriod;   //  on time of LED1 (usecs)  
  u16_t OffPeriod;  // off time of LED1 (usecs)
} ConfigLED_t;

//} Structures/ Enums
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

int main(void);
void initADC(volatile u16_t* ConvertedValue, u8_t AdcChannel);
void initPort(GPIO_TypeDef* GPIOx, u32_t Pin, GPIOMode_TypeDef Type);
void initPorts();

// task which spawns all other tasks + runs main eventloop
void taskMain(void *pvParameters);

// example LED controller
void taskLEDs(void *pvParameters);

void vApplicationMallocFailedHook(void);  // called in case pvPortMalloc() fails 

//} Function prototypes
//{ Function definitions *************************************************

        int main(void) {

    SystemInit();
    initPorts();
    
    xTaskCreate(taskMain,                          // function to start as thread
                (const signed char*) "taskMain",   // thread name (just for debugging)
                (size_t) configMINIMAL_STACK_SIZE, // stack size 
                (void *) 0,                        // passed as argument to taskLEDs()                       
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
               );
    
    vTaskStartScheduler(); // FreeRTOS-scheduler (never returns)
}
       void initADC(volatile u16_t* ConvertedValue, u8_t AdcChannel) {         // configure Analog Digital Converters
    
    /*{ Example code
    
    // DMA1 channel1 configuration
    DMA_DeInit(DMA1_Channel1);
    DMA_StructInit(&DMA_InitStructure);
    DMA_InitStructure.DMA_PeripheralBaseAddr = ADC1_DR_Address;
    DMA_InitStructure.DMA_MemoryBaseAddr     = (u32_t) ConvertedValue;
    DMA_InitStructure.DMA_DIR                = DMA_DIR_PeripheralSRC;
    DMA_InitStructure.DMA_BufferSize         = 1;
    DMA_InitStructure.DMA_PeripheralInc      = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc          = DMA_MemoryInc_Disable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize     = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode               = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority           = DMA_Priority_High;
    DMA_InitStructure.DMA_M2M                = DMA_M2M_Disable;
    DMA_Init(DMA1_Channel1, &DMA_InitStructure);
    
    // Enable DMA1 channel1
    DMA_Cmd(DMA1_Channel1, ENABLE);
    
    // ADC1 configuration
    ADC_InitStructure.ADC_Mode               = ADC_Mode_Independent;
    ADC_InitStructure.ADC_ScanConvMode       = ENABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
    ADC_InitStructure.ADC_ExternalTrigConv   = ADC_ExternalTrigConv_None;
    ADC_InitStructure.ADC_DataAlign          = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfChannel       = 1;
    ADC_Init(ADC1, &ADC_InitStructure);
    ADC_RegularChannelConfig(ADC1, AdcChannel, 1, ADC_SampleTime_55Cycles5);
    ADC_DMACmd(ADC1, ENABLE);                    // Enable ADC1 DMA
    ADC_Cmd(ADC1, ENABLE);                       // Enable ADC1
    
    }Example code*/
}
       void initPorts() {                                                      // basic initialisation of all used peripherals

    // enable clocks to all used ports (other clocks are enabled by initPort() automatically)
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO,  ENABLE);
#if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL)
    RCC_ADCCLKConfig(RCC_PCLK2_Div2);  // ADCCLK = PCLK2/2
#else
    RCC_ADCCLKConfig(RCC_PCLK2_Div4);  // ADCCLK = PCLK2/4
#endif
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1,   ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);

    

    /*{ Example code
    initPort(PB_LED,                 GPIO_Mode_Out_PP);
    initPort(PB_VALVES_HIGH_VOLTAGE, GPIO_Mode_Out_PP); portClr(PB_VALVES_HIGH_VOLTAGE);
    initPort(PB_PWM_MOTOR_FRESH_AIR, GPIO_Mode_Out_PP); portClr(PB_PWM_MOTOR_FRESH_AIR);
    initPort(PB_PWM_MOTOR_WARM_AIR,  GPIO_Mode_Out_PP); portSet(PB_PWM_MOTOR_WARM_AIR);
    initPort(PB_VALVE_GAS_HIGH,      GPIO_Mode_Out_PP); portSet(PB_VALVE_GAS_HIGH);
    initPort(PB_VALVE_GAS_LOW,       GPIO_Mode_Out_PP); portSet(PB_VALVE_GAS_LOW);

    initADC(&AIN_ParallelSwitches, ADC_Channel_15);
    ADC_ResetCalibration(ADC1);                  // Enable ADC1 reset calibaration register
    while(ADC_GetResetCalibrationStatus(ADC1));  // wait for end of ADC1 reset calibration register
    ADC_StartCalibration(ADC1);                  // Start ADC1 calibaration
    while(ADC_GetCalibrationStatus(ADC1));       // wait for end of ADC1 calibration
      
    // Start ADC1 Software Conversion 
    ADC_SoftwareStartConvCmd(ADC1, ENABLE);
    }Example code*/
}
       void vApplicationMallocFailedHook(void) {
  while (1);
}
       void taskMain(void *pvParameters) {
    
    ConfigLED_t ConfigLED = { PB_LED1, 100, 100 };
    xTaskCreate(taskLEDs,                          // function to start as thread
                (const signed char*) "taskLEDs",   // thread name (just for debugging)
                (size_t) configMINIMAL_STACK_SIZE, // stack size 
                (void *) &ConfigLED,               // passed as argument to taskLEDs()                       
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
               );
     
    while (1) {
      while (ConfigLED.OnPeriod > 0) {
        ConfigLED.OnPeriod--;
        mSleep(10);
      }
      while (ConfigLED.OnPeriod < 100) {
        ConfigLED.OnPeriod++;
        mSleep(10);
      }
    }
}
       void taskLEDs(void *pvParameters) {                                 // task: provides main user menu
  ConfigLED_t *ConfigLED = (ConfigLED_t*) pvParameters;
  if (ConfigLED != NULL) {
    
    initPort(ConfigLED->GPIOx, ConfigLED->Pin, GPIO_Mode_Out_PP);
    
    while (1) {
      portSet(ConfigLED->GPIOx, ConfigLED->Pin);
      uSleep(ConfigLED->OnPeriod);
      portClr(ConfigLED->GPIOx, ConfigLED->Pin);
      uSleep(ConfigLED->OffPeriod);
    }
  }
}

//} Function definitions
